package com.ye.back.controller;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ye.back.entity.Exam;
import com.ye.back.response.Result;
import com.ye.back.service.ExamService;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author hixon
 * @since 2023-03-28
 */
@RestController
@RequestMapping("/exam")
public class ExamController {

    @Autowired
    private ExamService examService;

    @ApiOperation("获取列表")
    @GetMapping("/list")
    public String list(String name, Integer schoolId) {
        LambdaQueryWrapper<Exam> queryWrapper = new QueryWrapper<Exam>().lambda();
        if (!StringUtils.isBlank(name)){
            queryWrapper.like(Exam::getName, name);
        }
        if (schoolId != null){
            queryWrapper.eq(Exam::getSchoolId, schoolId);
        }
        List<Exam> list = examService.list(queryWrapper);

        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }

    @ApiOperation("获取列表")
    @GetMapping("/getExamBySchoolId")
    public String getExamBySchoolId(String schoolId) {
        List<Exam> list = examService.list(new QueryWrapper<Exam>().lambda().eq(Exam::getSchoolId,schoolId));
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }

    @ApiOperation("新增")
    @PostMapping("/create")
    public String create(@RequestBody Exam exam) {
        boolean result = examService.save(exam);
        if (result){
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }else {
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
    }

    @ApiOperation("修改")
    @PostMapping("/update")
    public String update(@RequestBody Exam exam) {
        boolean result = examService.updateById(exam);
        if (result){
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }else {
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
    }

    @ApiOperation("删除")
    @DeleteMapping("/delete")
    public String delete(Integer id) {
        boolean result = examService.removeById(id);
        if (result){
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }else {
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
    }

    @GetMapping("/query")
    public String query(String name) {
        List<Exam> list = examService.list(new QueryWrapper<Exam>().lambda().eq(Exam::getName, name));
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }
}
