package com.ye.back.controller;


import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ye.back.dto.UserListener;
import com.ye.back.entity.User;
import com.ye.back.entity.UserRole;
import com.ye.back.response.Result;
import com.ye.back.service.UserRoleService;
import com.ye.back.service.UserService;
import com.ye.back.token.JwtToken;
import com.ye.back.util.JwtUtil;
import com.ye.back.util.StringUtil;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.Nullable;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 */
@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;

    @Autowired
    private UserRoleService userRoleService;


    /**
     * 自定义token登录
     *
     * @return
     */
    @ApiOperation("登录")
    @PostMapping("/login")
    public String login(@RequestBody User user) {
        if (user.getUsername() == null || user.getPassword() == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("账号密码不能为空"));
        }
        Subject subject = SecurityUtils.getSubject();
        String token = JwtUtil.createJWT(user.getUsername(), "back", "user", 1000 * 60 * 60 * 24);
        JwtToken jwtToken = new JwtToken(token, user.getPassword());
        try {
            subject.login(jwtToken);
        } catch (UnknownAccountException e) {
            return JSON.toJSONString(new Result().setCode(401).setMessage("账号不存在"));
        } catch (IncorrectCredentialsException e) {
            return JSON.toJSONString(new Result().setCode(402).setMessage("密码错误"));
        }
        User backUser = userService.getUserByUsername(user.getUsername());
        Map<String, Object> map = new HashMap<>();
        map.put("user", backUser);
        map.put("token", token);
        return JSON.toJSONString(new Result().setCode(200).setMessage("登录成功").setData(map));
    }

//    @ApiOperation("登录")
//    @RequestMapping(value = "/login", method = RequestMethod.POST)
//    public String login(@ApiParam("账号") String username, @ApiParam("密码") String password) {
//        System.out.println("登录");
//        Subject subject = SecurityUtils.getSubject();
//        UsernamePasswordToken token = new UsernamePasswordToken(username, password);
//        try {
//            subject.login(token);
//        } catch (UnknownAccountException e) {
//            return JSON.toJSONString(new Result().setCode(401).setMessage("账号不存在"));
//        } catch (IncorrectCredentialsException e) {
//            return JSON.toJSONString(new Result().setCode(402).setMessage("密码错误"));
//        }
//        User user = userService.getUserByUsername(username);
//        user.setPassword(null);
//        user.setSalt(null);
//        return JSON.toJSONString(new Result().setCode(200).setMessage("登录成功").setData(user));
//    }

    @ApiOperation("退出登录")
    @GetMapping("/logout")
    public Result logout() {
        SecurityUtils.getSubject().logout();
        return new Result().setCode(200).setMessage("成功退出");
    }

    //    @RequiresAuthentication
    @RequestMapping(value = "/getMenuList", method = RequestMethod.GET)
    public String getMenuList(String username) {
        if (StringUtils.isEmpty(username)) {
            return JSON.toJSONString(new Result().setCode(401).setMessage("未登录,请先登录"));
        }
        List<Map<String, Object>> list = userService.getMenuList(username);
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }


    @RequiresPermissions("person:look")
    @RequestMapping(value = "/lookPerson", method = RequestMethod.GET)
    public String getPerson(@Nullable String username, @Nullable String roleName, @Nullable String pageNum, @Nullable String pageSize) {
        if (StringUtils.isBlank(username)) {
            username = null;
        }
        if (StringUtils.isBlank(roleName)) {
            roleName = null;
        }
        Integer first = StringUtil.changeString(pageNum);
        Integer second = StringUtil.changeString(pageSize);
        Map<String, Object> list = userService.lookPerson(username, roleName, first, second);
        return JSON.toJSONString(new Result().setCode(200).setMessage("成功").setData(list));
    }

    @ApiOperation("获取学院所有老师")
    @GetMapping("/getTeachers")
    public String getTeachersBySchoolId(@Nullable Integer schoolId,@Nullable String name) {
        if (StringUtils.isBlank(name)) {
            name = null;
        }
        List<User> list = userService.getAllTeachersBySchoolId(schoolId,name);
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }

    @ApiOperation("获取学院所有学生")
    @GetMapping("/getStudents")
    public String getStudentsBySchoolId(Integer schoolId,String name) {
        if (StringUtils.isBlank(name)){
            name = null;
        }
        List<User> list = userService.getStudentsBySchoolId(schoolId,name);
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }

    @ApiOperation("获取所有教务")
    @GetMapping("/getAllJw")
    public String getAllJw() {
        List<User> list = userService.getAllJw();
        if (list == null) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("获取错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("获取成功").setData(list));
        }
    }

    @ApiOperation("修改用户信息")
    @PostMapping("/edit")
    public String edit(@RequestBody User user) {
        boolean result = userService.updateById(user);
        if (!result) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }
    }

    @ApiOperation("新增用户")
    @PostMapping("/create")
    public String create(@RequestBody User user) {
        user.setPassword(StringUtil.md5(user.getPassword()+"wcwad12"));
        user.setSalt("wcwad12");
        boolean result = userService.save(user);
        if (user.getStudentClass() != null){
            //学生
            UserRole userRole = new UserRole();
            userRole.setUserId(user.getUserId());
            userRole.setRoleId(1);
            userRoleService.save(userRole);
        }else {
            //老师
            UserRole userRole = new UserRole();
            userRole.setUserId(user.getUserId());
            userRole.setRoleId(2);
            userRoleService.save(userRole);
        }
        if (!result) {
            return JSON.toJSONString(new Result().setCode(500).setMessage("错误"));
        } else {
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }
    }

    @DeleteMapping("/delete")
    public String delete(Integer id) {
        boolean result = userService.removeById(id);
        if (result){
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }else {
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
    }

    @PostMapping("/changePwd")
    public String changePwd(@RequestBody User user) {
        user.setPassword(StringUtil.md5(user.getPassword()+"wcwad12"));
        boolean result = userService.lambdaUpdate().set(User::getPassword,user.getPassword()).eq(User::getUserId,user.getUserId()).update();
        if (result){
            return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
        }else {
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
    }

    @ApiOperation(value = "用户导入")
    @PostMapping("/import")
    public String userImport(@ApiParam(value = "文档", required = true) MultipartFile file) {
        try {
            if (file != null) {
                EasyExcel.read(file.getInputStream(), User.class, new UserListener(userService)).sheet().doRead();
            }
        } catch (IOException e) {
            e.printStackTrace();
            return JSON.toJSONString(new Result().setCode(500).setMessage("失败"));
        }
        return JSON.toJSONString(new Result().setCode(200).setMessage("成功"));
    }
}
